@@ -778,6 +778,8 @@ static vector_obj* gUserCompletions;
 static string_obj* gUserCompletionStatment;
 static sStatments* gUserCompletionStatments;
 
+char* readline_file_completion(const char* text, int rstat);
+
 char* readline_user_completion(const char* text, int stat)
 {
     static int index, wordlen;
@@ -785,64 +787,6 @@ char* readline_user_completion(const char* text, int stat)
     if(stat == 0) {
         wordlen = strlen(text);
         index = 0;
-
-        int k;
-        for(k=0; k<vector_size(gUserCompletions); k++) {
-            string_obj* str = vector_item(gUserCompletions, k);
-            string_delete(str);
-        }
-        vector_clear(gUserCompletions);
-
-        sWFd* pipeout = WFD_NEW(-1);
-        sRFd* pipein = RFD_NEW2(-1, string_c_str(gUserCompletionStatment));
-        string_delete(gUserCompletionStatment);
-
-        int rcode = saphire_run(gUserCompletionStatments, "user completion", pipeout, pipein,  2);
-        if(rcode >= 0) {
-            char* p = pipeout->mBuffer;
-            while(1) {
-                string_obj* str = STRING_NEW("");
-                while(1) {
-                    if(*p == 0) {
-                        break;
-                    }
-                    else if(*p == '\n') {
-                        p++;
-                        break;
-                    }
-                    else {
-                        string_push_back2(str, *p);
-                        p++;
-                    }
-                }
-
-                if(string_c_str(str)[0] != 0) {
-                    vector_add(gUserCompletions, str);
-                }
-                else {
-                    string_delete(str);
-                }
-
-                if(*p == 0) break;
-            }
-        }
-        else {
-            fprintf(stderr, "run time err\n");
-            fprintf(stderr, "%s", string_c_str(gErrMsg));
-
-            sWFd_delete(pipeout);
-            sRFd_delete(pipein);
-            return NULL;
-        }
-        sWFd_delete(pipeout);
-        sRFd_delete(pipein);
-    }
-
-    char* returned_variable = saphire_get_global_var("COMPLETION_ADD_SPACE");
-    if(returned_variable && strcmp(returned_variable, "0") == 0) {
-        rl_completion_append_character= 0;
-    } else {
-        rl_completion_append_character= ' ';
     }
 
     while(index < vector_size(gUserCompletions)) {
@@ -1448,23 +1392,20 @@ char** readline_on_complete(const char* text, int start, int end)
             p++;
         }
     }
-
-//printf("command %s buf %s\n", string_c_str(command), string_c_str(buf));
-
-    sStatments* statments;
-    /// 最後の単語入力が=が入っている場合ファイル名の補完をしたいのでカスタマイズ補完は無効にしたい
+/*
+    /// 最後の単語入力に=が入っている場合ファイル名の補完がしたい
     if(strstr(string_c_str(buf), "=")) {
-        statments = NULL;
-    }
-    else {
-        statments = hash_item(gUserCompRegex, string_c_str(command));
+        file = TRUE;
     }
+*/
 
     /// コマンドの補完でもファイル名に/が入っている場合はファイル名の補完にしたい
     if(strstr(string_c_str(command), "/")) {
         program = FALSE;
     }
 
+    sStatments* statments = hash_item(gUserCompRegex, string_c_str(command));
+
     string_delete(command);
     string_delete(buf);
     int i;
@@ -1507,8 +1448,78 @@ char** readline_on_complete(const char* text, int start, int end)
         return rl_completion_matches(text, readline_program_completion);
     }
     else if(statments) {
-        gUserCompletionStatment = statment;
-        gUserCompletionStatments = statments;
+        saphire_set_global_var("COMPLETION_ADD_SPACE", "1");
+
+        int k;
+        for(k=0; k<vector_size(gUserCompletions); k++) {
+            string_obj* str = vector_item(gUserCompletions, k);
+            string_delete(str);
+        }
+        vector_clear(gUserCompletions);
+
+        sWFd* pipeout = WFD_NEW(-1);
+        sRFd* pipein = RFD_NEW2(-1, string_c_str(statment));
+        string_delete(statment);
+
+        int rcode = saphire_run(statments, "user completion", pipeout, pipein,  2);
+        if(rcode == 0) {
+            char* p = pipeout->mBuffer;
+            while(1) {
+                string_obj* str = STRING_NEW("");
+                while(1) {
+                    if(*p == 0) {
+                        break;
+                    }
+                    else if(*p == '\n') {
+                        p++;
+                        break;
+                    }
+                    else {
+                        string_push_back2(str, *p);
+                        p++;
+                    }
+                }
+
+                if(string_c_str(str)[0] != 0) {
+                    vector_add(gUserCompletions, str);
+                }
+                else {
+                    string_delete(str);
+                }
+
+                if(*p == 0) break;
+            }
+        }
+        else if(rcode > 0) {
+            sWFd_delete(pipeout);
+            sRFd_delete(pipein);
+
+#if defined(HAVE_MIGEMO_H)
+            char** str = rl_completion_matches(text, readline_file_completion_migemo);
+            if(gMigemoReg) { onig_free(gMigemoReg); gMigemoReg = NULL; }
+            return str;
+#else
+            return rl_completion_matches(text, readline_file_completion);
+#endif
+        }
+        else {
+            fprintf(stderr, "run time err\n");
+            fprintf(stderr, "%s", string_c_str(gErrMsg));
+
+            sWFd_delete(pipeout);
+            sRFd_delete(pipein);
+            return NULL;
+        }
+        sWFd_delete(pipeout);
+        sRFd_delete(pipein);
+
+        char* returned_variable = saphire_get_global_var("COMPLETION_ADD_SPACE");
+        if(returned_variable && strcmp(returned_variable, "0") == 0) {
+            rl_completion_append_character= 0;
+        } else {
+            rl_completion_append_character= ' ';
+        }
+
         return rl_completion_matches(text, readline_user_completion);
     }
 #if defined(HAVE_MIGEMO_H)
@@ -1746,7 +1757,7 @@ BOOL commands_completion(int* rcode, vector_obj* argv, vector_obj* blocks, vecto
     return TRUE;
 }
 
-static void read_rc_obj_file(BOOL run_user_runtimescript)
+static void read_rc_obj_file()
 {
     char rc_fname[PATH_MAX];
     snprintf(rc_fname, PATH_MAX, "%s/completion.sao", SYSCONFDIR);
@@ -1774,28 +1785,31 @@ static void read_rc_obj_file(BOOL run_user_runtimescript)
     }
 
     /// ユーザーのランタイムスクリプト ////
-    if(run_user_runtimescript) {
-        char* user_rc_fname = getenv("SAPHIRE_COMPL_USERRC");
-        if(user_rc_fname) {
-            if(access(user_rc_fname, R_OK) == 0) {
-                sWFd* pipeout = WFD_NEW(STDOUT_FILENO);
-                sRFd* pipein = RFD_NEW(STDIN_FILENO);
-                saphire_init_stack_frame();
-                if(saphire_load_obj(user_rc_fname
-                    , pipeout, pipein, STDERR_FILENO, NULL) < 0)
-                {
-                    fprintf(stderr, "%s", string_c_str(gErrMsg));
-                    exit(1);
-                }
-                sWFd_flash(pipeout);
-                sRFd_delete(pipein);
-                sWFd_delete(pipeout);
+    char* user_rc_fname = getenv("SAPHIRE_COMPL_USERRC");
+    if(user_rc_fname) {
+        if(access(user_rc_fname, R_OK) == 0) {
+            sWFd* pipeout = WFD_NEW(STDOUT_FILENO);
+            sRFd* pipein = RFD_NEW(STDIN_FILENO);
+            saphire_init_stack_frame();
+            if(saphire_load_obj(user_rc_fname
+                , pipeout, pipein, STDERR_FILENO, NULL) < 0)
+            {
+                fprintf(stderr, "%s", string_c_str(gErrMsg));
+                exit(1);
             }
+            sWFd_flash(pipeout);
+            sRFd_delete(pipein);
+            sWFd_delete(pipeout);
         }
     }
 }
 
-void readline_init(BOOL run_user_runtimescript)
+char* readline_on_complete_entry(const char* a, int b)
+{
+    return NULL;
+}
+
+void readline_init(BOOL runtime_script)
 {
     gUserCompRegex = HASH_NEW(100);
     gUserCompletions = VECTOR_NEW(50);
@@ -1809,6 +1823,7 @@ void readline_init(BOOL run_user_runtimescript)
     rl_filename_quote_characters = " \\\t\n\"|$*?[]<>{}&;#()'%";
     rl_completion_append_character= ' ';
     rl_char_is_quoted_p = (rl_linebuf_func_t*)lftp_char_is_quoted;
+    rl_completion_entry_function = readline_on_complete_entry;
 
 #if defined(HAVE_MIGEMO_H)
     migemo_init();
@@ -1829,8 +1844,9 @@ void readline_init(BOOL run_user_runtimescript)
     saphire_add_inner_command("remove_history", commands_remove_history);
     saphire_add_inner_command("completion", commands_completion);
     saphire_add_inner_command("call_completion", commands_call_completion);
-    if(run_user_runtimescript) {
-        read_rc_obj_file(TRUE);
+    char* read_rc_env = getenv("SAPHIRE_USER_COMPLETION");
+    if(runtime_script && read_rc_env && strcmp(read_rc_env, "1") == 0) {
+        read_rc_obj_file();
     }
 
     files = VECTOR_NEW(30);
