@@ -45,6 +45,7 @@ int str_termlen_range(enum eKanjiCode code, char* mbs, int utfpos1, int utfpos2)
                 mbs2[putfpos2 - putfpos1] = 0;
 
                 int result = str_termlen(kUtf8, mbs2);
+                FREE(mbs2);
 
                 return result;
             }
@@ -72,17 +73,25 @@ BOOL is_utf8_bytes(char* mbs)
     char* mbs2 = MALLOC(size);
 
     if(mbstowcs(wcs, mbs, size) < 0) {
+        FREE(wcs);
+        FREE(mbs2);
         return FALSE;
     }
 
     if(wcstombs(mbs2, wcs, size) < 0) {
+        FREE(wcs);
+        FREE(mbs2);
         return FALSE;
     }
 
     if(strcmp(mbs, mbs2) == 0) {
+        FREE(wcs);
+        FREE(mbs2);
         return TRUE;
     }
     else {
+        FREE(wcs);
+        FREE(mbs2);
         return FALSE;
     }
 }
@@ -118,6 +127,8 @@ int str_pointer2kanjipos(enum eKanjiCode code, char* mbs, char* point)
 
         result = str_kanjilen(code, mbs2);
 
+        FREE(mbs2);
+
         return result;
     }
 }
@@ -284,9 +295,11 @@ int str_termlen(enum eKanjiCode code, char* mbs)
 
         wcs = (wchar_t*)MALLOC(sizeof(wchar_t)*((strlen(mbs)+1)*MB_CUR_MAX));
         if(mbstowcs(wcs, mbs, (strlen(mbs)+1)*MB_CUR_MAX) < 0) {
+            FREE(wcs);
             return -1;
         }
         result = wcswidth(wcs, wcslen(wcs));
+        FREE(wcs);
         if(result < 0) {
             return strlen(mbs);
         }
@@ -321,6 +334,8 @@ int str_termlen2(enum eKanjiCode code, char* mbs, int pos)
             result = str_termlen(code, mbs2);
         }
 
+        FREE(mbs2);
+
         return result;
     }
     else {
