@@ -358,7 +358,7 @@ static void expand_tilda(string_obj* buf, string_obj* path)
 }
 
 /// 一単語読み込み
-static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled, int* read_end_of_statment, sCommand* command, char* sname, int* sline)
+static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled, int* read_end_of_statment, sCommand* command, char* sname, int* sline, BOOL expand_quote)
 {
     /// 単語の区切り記号かどうかのテーブル 127以下なら
     static unsigned char table[] = { 
@@ -422,31 +422,55 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
             (*p)+=2;
         }
         else if(!squote && !dquote && **p == '\\') {
-            string_push_back2(not_evaled, **p);
-            (*p)++;
+            if(expand_quote) {
+                string_push_back2(not_evaled, **p);
+                (*p)++;
 
-            if(**p == 0) {
-                err_msg("unexpected end. can't quote null.", sname, *sline);
-                return FALSE;
+                if(**p == 0) {
+                    err_msg("unexpected end. can't quote null.", sname, *sline);
+                    return FALSE;
+                }
+                else if(**p == '\n') {
+                    (*sline)++;
+                }
+
+                string_push_back2(not_evaled, **p);
+                string_push_back2(buf, **p);
+
+                (*p)++;
             }
-            else if(**p == '\n') {
-                (*sline)++;
+            else {
+                string_push_back2(not_evaled, **p);
+                string_push_back2(buf, **p);
+
+                (*p)++;
             }
-
-            string_push_back2(not_evaled, **p);
-            string_push_back2(buf, **p);
-
-            (*p)++;
         }
         else if(!dquote && **p == '\'') {
-            string_push_back2(not_evaled, **p);
-            (*p)++;
-            squote = !squote;
+            if(expand_quote) {
+                string_push_back2(not_evaled, **p);
+                (*p)++;
+                squote = !squote;
+            }
+            else {
+                string_push_back2(not_evaled, **p);
+                string_push_back2(buf, **p);
+
+                (*p)++;
+            }
         }
         else if(!squote && **p == '"') {
-            dquote = !dquote;
-            string_push_back2(not_evaled, **p);
-            (*p)++;
+            if(expand_quote) {
+                dquote = !dquote;
+                string_push_back2(not_evaled, **p);
+                (*p)++;
+            }
+            else {
+                string_push_back2(not_evaled, **p);
+                string_push_back2(buf, **p);
+
+                (*p)++;
+            }
         }
         /// コマンドライン展開
         else if(!squote && **p == '$') {
@@ -498,15 +522,96 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
             (*p)++;
 
             if(strlen(string_c_str(buf)) == 1 && command) {
-                if(!read_one_argument(p, buf, not_evaled, read_end_of_statment, NULL, sname, sline))
+                if(!read_one_argument(p, buf, not_evaled, read_end_of_statment, NULL, sname, sline, FALSE))
                 {
                     return FALSE;
                 }
 
                 string_obj* path = STRING_NEW("");
                 expand_tilda(buf, path);
-                vector_add(command->mArgs, ARG_NEW(0, MANAGED path));
-                string_put(buf, ""); // チルダ展開のパターンはmArgsに追加しない
+
+                string_obj* path2 = STRING_NEW("");
+                BOOL glob_ = FALSE;
+                char* p = string_c_str(path);
+                BOOL squote = FALSE;
+                BOOL dquote = FALSE;
+                while(*p) {
+                    if(!dquote && *p == '\'') {
+                        p++;
+                        squote = !squote;
+                    }
+                    else if(!squote && *p == '"') {
+                        p++;
+                        dquote = !dquote;
+                    }
+                    else if(squote || dquote) {
+                        string_push_back2(path2, *p);
+                        p++;
+                    }
+                    else if(*p == '\\') {
+                        p++;
+                        string_push_back2(path2, *p);
+                        p++;
+                    }
+                    else if(*p == '[') {
+                        string_push_back2(path2, *p);
+                        p++;
+
+                        while(*p) {
+                            if(*p == ']') {
+                                glob_ = TRUE;
+                                string_push_back2(path2, *p);
+                                p++;
+                                break;
+                            }
+                            else {
+                                string_push_back2(path2, *p);
+                                p++;
+                            }
+                        }
+                    }
+                    else if(*p == '?' || *p == '*') {
+                        glob_ = TRUE;
+                        string_push_back2(path2, *p);
+                        p++;
+                    }
+                    else {
+                        string_push_back2(path2, *p);
+                        p++;
+                    }
+                }
+
+                string_delete(path);
+
+                if(glob_) {
+                    glob_t result;
+                    int rc = glob(string_c_str(path2), 0, NULL, &result);
+                    if(rc == GLOB_NOSPACE) {
+                        err_msg("read_one_argument: out of space during glob operation", sname, *sline);
+                        string_delete(path2);
+                        return FALSE;
+                    }
+                    else if(rc == GLOB_NOMATCH) {
+                    }
+                    else {
+                        int i;
+                        for(i=0; i<result.gl_pathc; i++) {
+                            char* file = result.gl_pathv[i];
+                            if(strcmp(file, ".") != 0 && strcmp(file, "..") != 0)
+                            {
+                                string_obj* str = STRING_NEW(file);
+                                vector_add(command->mArgs, ARG_NEW(0, MANAGED str));
+                            }
+                        }
+
+                        string_put(buf, ""); // グロブのパターンはmArgsに追加しない
+                    }
+                    string_delete(path2);
+                }
+                else {
+                    vector_add(command->mArgs, ARG_NEW(0, MANAGED path2));
+                    string_put(buf, ""); // チルダ展開のパターンはmArgsに追加しない
+                }
                 break;
             }
         }
@@ -517,7 +622,7 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
             (*p)++;
 
             if(command) {
-                if(!read_one_argument(p, buf, not_evaled, read_end_of_statment, NULL, sname, sline))
+                if(!read_one_argument(p, buf, not_evaled, read_end_of_statment, NULL, sname, sline, TRUE))
                 {
                     return FALSE;
                 }
@@ -964,7 +1069,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
     while(**p) {
         string_obj* buf = STRING_NEW("");
 
-        if(!read_one_argument(p, buf, not_evaled, read_end_of_statment, command, sname, sline)) 
+        if(!read_one_argument(p, buf, not_evaled, read_end_of_statment, command, sname, sline, TRUE)) 
         {
             string_delete(buf);
             return FALSE;
