@@ -21,40 +21,21 @@ typedef struct
 //////////////////////////////////////////////////////////////////////
 
 // 初期化
+#if defined(MDEBUG)
+string_obj* string_new_debug(char* str, const char* fname, int line, const char* func_name);
+#define STRING_NEW(str) string_new_debug(str, __FILE__, __LINE__, __FUNCTION__)
+#else
 string_obj* string_new(char* str);
 #define STRING_NEW(str) string_new(str)
+#endif
 
-
-void string_delete(string_obj* str);                    // 解放
+void string_delete(string_obj* self);
 
 #define string_length(o) ((string_obj*)o)->mLen
 #define string_size(o) ((string_obj*)o)->mLen
 
-//int string_length(string_obj* obj);                     // 文字列数を返す
-//int string_size(string_obj* obj);                       // 文字列数を返す
-/*
-//////////////////////////////////////////////////////////////////////
-// 文字列数を返す
-//////////////////////////////////////////////////////////////////////
-int string_length(string_obj* obj)
-{
-    return obj->mLen;
-}
-*/
-char* string_c_str(string_obj* obj);                    // c文字列のポインタを返す
-
 #define string_c_str(o) ((string_obj*)o)->mStr
 
-/*
-//////////////////////////////////////////////////////////////////////
-// 文字列数を返す
-//////////////////////////////////////////////////////////////////////
-char* string_c_str(string_obj* obj)
-{
-    return obj->mStr;
-}
-*/
-
 void string_push_back(string_obj* self, char* key);     // 後ろから文字列を追加
 void string_push_back2(string_obj* self, char key);     // 後ろから文字を追加
 void string_push_back3(string_obj* self, char* str, int n);
